"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var instant_1 = require("../excute/instant");
var list_1 = require("../excute/list");
var check_1 = require("../util/check");
var combineStartedToEnd = function (list) {
    return {
        type: 'str',
        va: list.join(' '),
    };
};
var combineString = function (args) {
    var started = false;
    var temp = [];
    var re = [];
    loop: for (var _i = 0, args_1 = args; _i < args_1.length; _i++) {
        var i = args_1[_i];
        if (i.type === 'comstart') {
            started = true;
            temp.push(i.va.substring(1, i.va.length));
            continue loop;
        }
        if (i.type === 'comend') {
            if (started) {
                started = false;
                temp.push(i.va.substring(0, i.va.length - 1));
                re.push(combineStartedToEnd(temp));
                temp = [];
            }
            continue loop;
        }
        if (started) {
            if (i.type === 'str' || i.type === 'var' || i.type === 'emp' || i.type === 'num') {
                temp.push(i.va);
            }
            else {
                throw new Error('combine failed exception');
            }
        }
        else {
            re.push(i);
        }
    }
    return re;
};
var parseArg = function (arg) {
    var regExpString = /^'([A-Z]|[a-z]|[0-9])*'$/;
    var regExpVariable = /^([A-Z]|[a-z])([A-Z]|[a-z]|[0-9])*$/;
    var regExpNumber = /^[0-9]+(.[0-9]+)?$/;
    var regExpExpression = /^(=|\+|-|\*|\/|<|>|<=|>=|==|!=)$/;
    var regExpEmpty = /^$/;
    var regExpCombineable = /^'([A-Z]|[a-z]|[0-9])*$/;
    var regExpCombineend = /^([A-Z]|[a-z]|[0-9])*'$/;
    if (regExpString.test(arg)) {
        return {
            type: 'str',
            va: arg.substring(1, arg.length - 1),
        };
    }
    else if (regExpVariable.test(arg)) {
        return {
            type: 'var',
            va: arg,
        };
    }
    else if (regExpNumber.test(arg)) {
        return {
            type: 'num',
            va: parseFloat(arg),
        };
    }
    else if (regExpExpression.test(arg)) {
        return {
            type: 'exp',
            va: arg,
        };
    }
    else if (regExpCombineable.test(arg)) {
        return {
            type: 'comstart',
            va: arg,
        };
    }
    else if (regExpCombineend.test(arg)) {
        return {
            type: 'comend',
            va: arg,
        };
    }
    else if (regExpEmpty.test(arg)) {
        return {
            type: 'emp',
            va: arg,
        };
    }
    else {
        return {
            type: 'err',
            va: arg,
        };
    }
};
var ast = function (code, optionsE) {
    var options = check_1.fixOption(optionsE);
    var regExpEnter = /\r\n|\r|\n/;
    var splited = code.split(regExpEnter).filter(function (line) { return Boolean(line.trim()); }).map(function (line) { return line.trim(); });
    var vars = [];
    return splited.map(function (value) {
        var dots = value.split(' ').map(function (dot) { return dot.trim(); });
        var command = dots.shift() || 'skip';
        var type = 'error';
        if (command === 'var') {
            var testShift = dots.shift();
            if (testShift) {
                command = testShift;
                if (list_1.internalList.indexOf(testShift) !== -1
                    || options.externals.map(function (external) { return external.command; }).indexOf(testShift) !== -1
                    || options.instants.map(function (instants) { return instants.command; }).indexOf(testShift) !== -1
                    || options.vars.map(function (singleVar) { return singleVar.name; }).indexOf(testShift) !== -1
                    || list_1.reservedWordList.indexOf(testShift) !== -1
                    || instant_1.instantList.indexOf(testShift) !== -1) {
                    type = 'error';
                }
                else {
                    vars.push(command);
                    type = 'assign';
                }
            }
        }
        else if (command === 'if') {
            type = 'if';
        }
        else if (command === 'for') {
            type = 'for';
        }
        else if (command === 'end') {
            type = 'end';
        }
        else if (vars.indexOf(command) !== -1) {
            type = 'assign';
        }
        else if (command === 'skip') {
            type = 'skip';
        }
        else {
            type = 'command';
        }
        var re = {
            type: type,
            val: command,
            args: combineString(dots.map(parseArg)),
        };
        return re;
    });
};
exports.default = ast;
