"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var deepclone_1 = require("../util/deepclone");
var expr_1 = require("./expr");
var list_1 = require("./list");
var check_1 = require("../util/check");
var instant_1 = require("./instant");
var findVar = function (val, vars) {
    for (var i = 0; i < vars.length; i++) {
        if (vars[i].name === val) {
            return i;
        }
    }
    return -1;
};
var excuteExpr = function (args, options, previous) {
    var current = args.shift();
    var vars = options.vars;
    var externalInstants = options.instants;
    if (!current) {
        return previous;
    }
    switch (current.type) {
        case 'exp':
            switch (current.va) {
                case '=':
                    return excuteExpr(args, options, previous);
                default:
                    var value = expr_1.default(current.va, previous, excuteExpr(args, options));
                    return excuteExpr(args, options, value);
            }
        case 'num':
        case 'str':
            return excuteExpr(args, options, current.va);
        case 'var':
            var instantIndex = instant_1.instantList.indexOf(current.va);
            if (instantIndex !== -1) {
                var result = void 0;
                try {
                    result = instant_1.instants[instantIndex].func(excuteExpr(args, options, previous));
                }
                catch (err) {
                    throw new Error('instant function excute failed');
                }
                return result;
            }
            var externalInstantIndex = externalInstants.map(function (externalInstant) { return externalInstant.command; }).indexOf(current.va);
            if (externalInstantIndex !== -1) {
                var result = void 0;
                try {
                    result = externalInstants[externalInstantIndex].func(excuteExpr(args, options, previous));
                }
                catch (err) {
                    throw new Error('external instant function excute failed');
                }
                return result;
            }
            var varIndex = findVar(current.va, vars);
            if (varIndex === -1) {
                throw new Error('undefined variable exception');
            }
            else {
                return excuteExpr(args, options, vars[varIndex].value);
            }
        case 'emp':
            return excuteExpr(args, options, previous);
        case 'err':
            throw new Error('unexpect argument exception');
    }
};
var excuteRecursive = function (astE, reE, options) {
    var ast = deepclone_1.deepCloneArray(astE);
    var re = deepclone_1.deepCloneArray(reE);
    var current = ast.shift();
    var vars = options.vars;
    if (!current) {
        return re;
    }
    loop: switch (current.type) {
        case 'if':
            if (!Boolean(excuteExpr(current.args, options))) {
                for (var _i = 0, ast_1 = ast; _i < ast_1.length; _i++) {
                    var i = ast_1[_i];
                    if (i.type !== 'end') {
                        i.type = 'skip';
                    }
                    else {
                        break loop;
                    }
                }
                throw new Error('end command not matched exception');
            }
            break;
        case 'for':
            throw new Error('for loop is not developed yet');
        case 'assign':
            var varIndex = findVar(current.val, vars);
            if (varIndex !== -1) {
                vars[varIndex].value = excuteExpr(current.args, options);
            }
            else {
                var currentVar = {
                    name: current.val,
                    value: excuteExpr(current.args, options),
                };
                vars.push(currentVar);
            }
            break;
        case 'command':
            if (list_1.internalList.indexOf(current.val)) {
                var currentCommand = {
                    type: 'internal',
                    value: current.val,
                    arg: excuteExpr(current.args, options),
                };
                re.push(currentCommand);
            }
            break;
        case 'error':
            throw new Error('unexpect namespace exception');
        case 'skip':
        default:
            break;
    }
    return excuteRecursive(ast, re, options);
};
var excute = function (ast, optionsE) {
    var options = check_1.fixOption(optionsE);
    return excuteRecursive(ast, [], options);
};
exports.default = excute;
